// fragment shader
#version 330
//in vec3 worldPosition;
layout(location = 0) out vec4 fragColor;
layout(location = 1) out vec4 brightColor;

uniform vec3 viewPosition;

uniform samplerCube depthMap;
uniform float far_plane;

varying vec3 worldPosition;
varying vec3 vnormal;
varying vec2 vuv;

struct Light {
	vec3 position;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

uniform Light light;
uniform sampler2D texture2d;


//uniform vec3 offset[7] = vec3 [] (vec3(0.0,0.0,0.0),vec3(1.0,0.0,0.0),vec3(-1.0,0.0,0.0),vec3(0.0,1.0,0.0),vec3(0.0,-1.0,0.0),vec3(0.0,0.0,1.0),vec3(0.0,0.0,1.0));
vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1, 1), vec3(1, -1, 1), vec3(-1, -1, 1), vec3(-1, 1, 1), 
   vec3(1, 1, -1), vec3(1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1, 0), vec3(1, -1, 0), vec3(-1, -1, 0), vec3(-1, 1, 0),
   vec3(1, 0, 1), vec3(-1, 0, 1), vec3(1, 0, -1), vec3(-1, 0, -1),
   vec3(0, 1, 1), vec3(0, -1, 1), vec3(0, -1, -1), vec3(0, 1, -1)
);

float ShadowCalculation(vec3 fragPosition) {
	
	vec3 fragToLight = fragPosition - light.position;
	//float closestDepth = texture(depthMap, fragToLight).r;
	//closestDepth *= far_plane;
	float currentDepth = length(fragToLight);

	float bias = 1.5;
	float shadow = 0.0;
	int samples = 20;
	float viewDistance = length(worldPosition - worldPosition);
	float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;
	
	for (int i = 0; i < samples; i++) {
		float closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
		closestDepth *= far_plane;
		if (currentDepth - bias > closestDepth) 
			shadow += 1.0;
	}
	
	//float shadow = (currentDepth - bias) > closestDepth ? 1.0 : 0.0;
	shadow /= float(samples);
	return shadow;
}


void main() {
	
	vec3 norm = normalize(vnormal);
	vec3 lightDir = normalize(light.position - worldPosition);

	fragColor = vec4(texture(texture2d,vuv).rgb,1.0f);//vec4(vuv,0.0,1.0);//

	//ambient and diffuse color
	float diff = max(dot(norm, lightDir), 0.0);
	float shadow = 0.0;

	/*
	float dist = length(worldPosition - light.position);
	for(int i = 0; i < 7; i++){
		vec3 c = offset[i] / 100.0 * dist;
		shadow += ShadowCalculation(worldPosition + c);
	}
	shadow = shadow / 7;
	*/
	shadow = ShadowCalculation(worldPosition);

	// light.ambient + 
	fragColor = vec4(light.ambient,1.0f) + (1.0 - shadow) * (diff * fragColor); //vec4(light.diffuse, 1.0) * (diff * fragColor);
	// for shadow mapping:         ^  - shadow //insert here
	
	fragColor = vec4(fragColor.rgb,1.0f);
	brightColor = vec4(0.0,0.0,0.0,1.0);
	//brightness output, with constant threshold
	float brightness = dot(fragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
	if (brightness >  1.0)
		brightColor = fragColor;
}