//Phong illumination fragment shader
#version 330

layout (location = 0) out vec4 fragcolor;
layout (location = 1) out vec4 brightColor;

varying vec3 fragPos;
varying vec3 vnormal;
varying vec2 _uv;


uniform float far_plane;


struct Material {
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
	float shininess;
};

struct Light {
	vec3 position;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
};

uniform Material material;
uniform Light light;
uniform vec3 viewPosition;
uniform mat4 model;

uniform float alpha;

uniform sampler2D texture2d;
uniform samplerCube depthMap;
	
//uniform vec3 offset[7] = vec3 [] (vec3(0.0,0.0,0.0),vec3(1.0,0.0,0.0),vec3(-1.0,0.0,0.0),vec3(0.0,1.0,0.0),vec3(0.0,-1.0,0.0),vec3(0.0,0.0,1.0),vec3(0.0,0.0,1.0));
vec3 gridSamplingDisk[20] = vec3[]
(
   vec3(1, 1, 1), vec3(1, -1, 1), vec3(-1, -1, 1), vec3(-1, 1, 1), 
   vec3(1, 1, -1), vec3(1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
   vec3(1, 1, 0), vec3(1, -1, 0), vec3(-1, -1, 0), vec3(-1, 1, 0),
   vec3(1, 0, 1), vec3(-1, 0, 1), vec3(1, 0, -1), vec3(-1, 0, -1),
   vec3(0, 1, 1), vec3(0, -1, 1), vec3(0, -1, -1), vec3(0, 1, -1)
);

float ShadowCalculation(vec3 fragPosition) {
	
	vec3 fragToLight = fragPosition - light.position;
	//float closestDepth = texture(depthMap, fragToLight).r;

	//closestDepth *= far_plane;
	float currentDepth = length(fragToLight);

	float shadow = 0.0;
	float bias = 1.5;
	int samples = 20;
	float viewDistance = length(viewPosition - fragPos);
	float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0;

	for (int i=0; i < samples; i++) {
		float closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
		closestDepth *= far_plane;
		if (currentDepth - bias > closestDepth) 
			shadow += 1.0;
	}
	shadow /= float(samples);


	//float shadow = (currentDepth - bias) > closestDepth ? 1.0 : 0.0;
	return shadow;
}

void main() {

	//prepare vectors
	vec3 norm = normalize(vnormal);
	vec3 lightdir = normalize(light.position - fragPos);

	vec3 viewdir = normalize(viewPosition - fragPos);
	vec3 reflectdir = reflect(-lightdir, norm);

	//components: ambient, diffuse, specular
	vec3 ambient = light.ambient * material.ambient;
	
	float diff = max(dot(norm, lightdir), 0.0);
	vec3 diffuse = light.diffuse * (diff * material.diffuse);
	
	float spec = pow(max(dot(viewdir, reflectdir), 0.0), material.shininess);
	vec3 specular = light.specular * (spec * material.specular);
	
	float shadow = ShadowCalculation(fragPos);
	/*
	float dist = length(fragPos - light.position);
	for(int i = 0; i < 7; i++){
		vec3 c = offset[i] / 100.0 * dist;
		shadow += ShadowCalculation(fragPos + c);
	}
	shadow = shadow / 7;
	*/

	//+ specular + ambient/object color
	fragcolor = vec4(ambient + (1.0 -shadow) *(specular + diffuse) , alpha); // + (1.0 - shadow) * (specular + diffuse)
	// for shadow mapping:        ^  - shadow //insert here
	
	brightColor = vec4(0.0,0.0,0.0,1.0);
	//brightness output, with constant threshold
	float brightness = dot(fragcolor.rgb, vec3(0.2126, 0.7152, 0.0722));
	if (brightness >  0.7)
		brightColor = fragcolor;

}